@@ -1,4 +1,4 @@
-__version__ = '1.0.1'
+__version__ = '1.1.0'
 
 import requests
 import json
@@ -17,19 +17,20 @@ class StreamBotConfig:
         self.user = user
 
 class StreamBot:
-    def __init__(self, api_key, bot_name, genesis_prompt, config=None, openai_url = 'https://api.openai.com/v1/chat/completions', model="gpt-3.5-turbo-0301"):
+    def __init__(self, api_key, bot_name, genesis_prompt=None, config=None, openai_url = 'https://api.openai.com/v1/chat/completions', model="gpt-3.5-turbo-0301", verbosity = 0):
         self.api_key = api_key
         self.bot_name = bot_name
         self.config = config or StreamBotConfig()
         self.genesis_prompt = genesis_prompt
-        self.messages = [{"role":"system", "content": genesis_prompt}]
+        self.messages = [{"role":"system", "content": genesis_prompt}] if genesis_prompt else []
         self.openai_url = openai_url
         self.model = model
+        self.verbosity = verbosity
 
-    def build_req_body(self):
+    def build_req_body(self, messages):
       req_body = {
           'model': self.model,
-          'messages': self.messages,
+          'messages': self.messages if messages == [] else messages,
           'stream': True
       }
   
@@ -51,13 +52,17 @@ class StreamBot:
         req_body['logit_bias'] = self.config.logit_bias
       if self.config.user is not None:
         req_body['user'] = self.config.user
+
+      if self.verbosity >= 3:
+        print(f"Current Request Body: {req_body}")
+      
       return req_body
 
     def add_message(self, message, role="user"):
       self.messages.append({"role": role, "content": message})
 
     def chat(self, messages = []):
-        reqBody = self.build_req_body()
+        reqBody = self.build_req_body(messages)
         reqHeaders = {
             'Accept': 'text/event-stream',
             'Authorization': 'Bearer ' + self.api_key
@@ -72,6 +77,8 @@ class StreamBot:
             print(self.bot_name + ": ")
             for event in client.events():
                 try:
+                    if self.verbosity >= 3:
+                      print(f"Event Data: {json.loads(event.data)}")
                     # Wrapped in try because json.loads fails due to `choices` not being present in last event
                     data = json.loads(event.data)['choices'][0]
                     # In first event, `delta` doesn't exist, so we check if the ['delta']['content'] keys are present
@@ -79,13 +86,42 @@ class StreamBot:
                     if 'delta' in data and 'content' in data['delta'] and data['finish_reason'] != 'stop':
                         response_text.append(data['delta']['content'])
                         print(data['delta']['content'], end="", flush=True)
-                except json.decoder.JSONDecodeError:
+                except json.decoder.JSONDecodeError as e:
+                    if self.verbosity >= 3:
+                        print(f"JSON Decoder Error: {e}")
                     pass
         except requests.exceptions.RequestException as e:
-            print(f"An error occurred: {e}")
+            if self.verbosity >= 3:
+              print(f"Requests Error: {e}")
+            pass
         
         result = "".join(response_text)
       
         self.add_message(result, role="assistant")
         
         return result
+
+    def chat_stream(self, messages = []):
+      reqBody = self.build_req_body(messages)
+      reqHeaders = {
+          'Accept': 'text/event-stream',
+          'Authorization': 'Bearer ' + self.api_key
+      }
+      try:
+          # Fire off conversation message array to OpenAI  
+          response = requests.post(self.openai_url, stream=True, headers=reqHeaders, json=reqBody)
+          # Using the Server Sent Events library to support "Stream" of tokens to simulate the AI typing out
+          client = sseclient.SSEClient(response)
+          # Array to capture the tokens since `response` will no longer be consumable
+          for event in client.events():
+              try:
+                  # Wrapped in try because json.loads fails due to `choices` not being present in last event
+                  data = json.loads(event.data)['choices'][0]
+                  # In first event, `delta` doesn't exist, so we check if the ['delta']['content'] keys are present
+                  # Also check for ['finish_reason'] to break on last message
+                  if 'delta' in data and 'content' in data['delta'] and data['finish_reason'] != 'stop':
+                      yield data['delta']['content']
+              except json.decoder.JSONDecodeError:
+                  pass
+      except requests.exceptions.RequestException as e:
+          print(f"An error occurred: {e}")
\ No newline at end of file
